@@ -1,6 +1,6 @@
 /* Parse a string into an internal timestamp.
 
-   Copyright (C) 1995, 1997-1998, 2003-2004, 2007, 2009-2017 Free Software
+   Copyright (C) 1995, 1997-1998, 2003-2004, 2007, 2009-2021 Free Software
    Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -19,11 +19,13 @@
 #include <stdbool.h>
 #include <time.h>
 
-bool parse_datetime (struct timespec *, char const *, struct timespec const *);
+bool parse_datetime (struct timespec *restrict,
+                     char const *, struct timespec const *);
 
 /* parse_datetime2 flag: if set, print debug/progress information to STDERR */
 #define PARSE_DATETIME_DEBUG 1
 
 /* same as above, supporting additional flags */
-bool parse_datetime2 (struct timespec *, char const *, struct timespec const *,
+bool parse_datetime2 (struct timespec *restrict,
+                      char const *, struct timespec const *,
                       unsigned int flags, timezone_t, char const *);
